# The LearningOnline Network with CAPA
#
# $Id: lonproblemanalysis.pm,v 1.85 2004/06/04 21:42:18 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::lonproblemanalysis;

use strict;
use Apache::lonnet();
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::loncoursedata();
use Apache::lonstatistics;
use Apache::lonlocal;
use Apache::lonstathelpers();
use Apache::lonstudentsubmissions();
use HTML::Entities();
use Time::Local();
use Spreadsheet::WriteExcel();

my $plotcolors = ['#33ff00', 
                  '#0033cc', '#990000', '#aaaa66', '#663399', '#ff9933',
                  '#66ccff', '#ff9999', '#cccc33', '#660000', '#33cc66',
                  ]; 

my @SubmitButtons = ({ name => 'PrevProblemAnalysis',
                       text => 'Previous Problem' },
                     { name => 'ProblemAnalysis',
                       text => 'Analyze Problem Again' },
                     { name => 'NextProblemAnalysis',
                       text => 'Next Problem' },
                     { name => 'break'},
                     { name => 'SelectAnother',
                       text => 'Choose a different Problem' },
                     { name => 'ExcelOutput',
                       text => 'Produce Excel Output' });


sub BuildProblemAnalysisPage {
    my ($r,$c)=@_;
    #
    my %Saveable_Parameters = ('Status' => 'scalar',
                               'Section' => 'array',
                               'NumPlots' => 'scalar',
                               'AnalyzeAs' => 'scalar',
                               'AnalyzeOver' => 'scalar',
                               );
    &Apache::loncommon::store_course_settings('problem_analysis',
                                              \%Saveable_Parameters);
    &Apache::loncommon::restore_course_settings('problem_analysis',
                                                \%Saveable_Parameters);
    #
    &Apache::lonstatistics::PrepareClasslist();
    #
    $r->print(&CreateInterface());
    #
    my @Students = @Apache::lonstatistics::Students;
    #
    if (@Students < 1) {
        $r->print('<h2>There are no students in the sections selected</h2>');
    }
    #
    my @CacheButtonHTML = 
        &Apache::lonstathelpers::manage_caches($r,'Statistics','stats_status');
    $r->rflush();
    #
    my $problem_types = '(option|radiobutton|numerical)';
    if (exists($ENV{'form.problemchoice'}) && 
        ! exists($ENV{'form.SelectAnother'})) {
        foreach my $button (@SubmitButtons) {
            if ($button->{'name'} eq 'break') {
                $r->print("<br />\n");
            } else {
                $r->print('<input type="submit" name="'.$button->{'name'}.'" '.
                          'value="'.&mt($button->{'text'}).'" />');
                $r->print('&nbsp;'x5);
            }
        }
        foreach my $html (@CacheButtonHTML) {
            $r->print($html.('&nbsp;'x5));
        }
        #
        $r->print('<hr />');
        $r->rflush();
        #
        # Determine which problem we are to analyze
        my $current_problem = &Apache::lonstathelpers::get_target_from_id
            ($ENV{'form.problemchoice'});
        #
        my ($prev,$curr,$next) = 
            &Apache::lonstathelpers::get_prev_curr_next($current_problem,
                                                        $problem_types,
                                                        'response',
                                                        );
        if (exists($ENV{'form.PrevProblemAnalysis'}) && defined($prev)) {
            $current_problem = $prev;
        } elsif (exists($ENV{'form.NextProblemAnalysis'}) && defined($next)) {
            $current_problem = $next;
        } else {
            $current_problem = $curr;
        }
        #
        # Store the current problem choice and send it out in the form
        $ENV{'form.problemchoice'} = 
            &Apache::lonstathelpers::make_target_id($current_problem);
        $r->print('<input type="hidden" name="problemchoice" value="'.
                  $ENV{'form.problemchoice'}.'" />');
        #
        if (! defined($current_problem->{'resource'})) {
            $r->print('resource is undefined');
        } else {
            my $resource = $current_problem->{'resource'};
            $r->print('<h1>'.$resource->{'title'}.'</h1>');
            $r->print('<h3>'.$resource->{'src'}.'</h3>');
            $r->print(&Apache::lonstathelpers::render_resource($resource));
            $r->rflush();
            my %Data = &Apache::lonstathelpers::get_problem_data
                ($resource->{'src'});
            my $ProblemData = $Data{$current_problem->{'part'}.
                                    '.'.
                                    $current_problem->{'respid'}};
            if ($current_problem->{'resptype'} eq 'option') {
                &OptionResponseAnalysis($r,$current_problem,
                                        $ProblemData,
                                        \@Students);
            } elsif ($current_problem->{'resptype'} eq 'radiobutton') {
                &RadioResponseAnalysis($r,$current_problem,
                                       $ProblemData,
                                       \@Students);
            } elsif ($current_problem->{'resptype'} eq 'numerical') {
                ## 
                ## analyze all responses of a problem at once
                my $res = $current_problem->{'resource'};
                foreach my $partid (@{$res->{'parts'}}) {
                    $current_problem->{'part'} = $partid;
                    foreach my $respid (@{$res->{'partdata'}->{$partid}->{'ResponseIds'}}) {
                        $current_problem->{'respid'}=$respid;
                        &NumericalResponseAnalysis($r,$current_problem,
                                                   $ProblemData,\@Students);
                    }
                }
            } else {
                $r->print('<h2>This analysis is not supported</h2>');
            }
        }
        $r->print('<hr />');
    } else {
        $r->print('<input type="submit" name="ProblemAnalysis" value="'.
                  &mt('Analyze Problem').'" />');
        $r->print('&nbsp;'x5);
        $r->print('<h3>'.&mt('Please select a problem to analyze').'</h3>');
        $r->print(&Apache::lonstathelpers::ProblemSelector
                  ($problem_types));
    }
}


#########################################################
#########################################################
##
##      Numerical Response Routines
##
#########################################################
#########################################################
sub NumericalResponseAnalysis {
    my ($r,$problem,$ProblemData,$Students) = @_;
    my $c = $r->connection();
    my ($resource,$partid,$respid) = ($problem->{'resource'},
                                      $problem->{'part'},
                                      $problem->{'respid'});
    #
    if (scalar(@{$resource->{'parts'}})>1) {
        if (@{$resource->{'partdata'}->{$partid}->{'ResponseIds'}}>1) {
            $r->print('<h3>'.
                      &mt('Part [_1], response [_2].',$partid,$respid).
                      '</h3>');
        } else { 
            $r->print('<h3>'.
                      &mt('Part [_1]',$partid,$respid).
                      '</h3>');
        }
    } elsif (@{$resource->{'partdata'}->{$partid}->{'ResponseIds'}}>1) {
        $r->print('<h3>'.&mt('Response [_1]',$respid).'</h3>');
    }
    #
    my $analysis_html;
    my $PerformanceData = &Apache::loncoursedata::get_response_data
        (\@Apache::lonstatistics::SelectedSections,
         $Apache::lonstatistics::enrollment_status,
         $resource->{'symb'},$respid);
    if (! defined($PerformanceData) || 
        ref($PerformanceData) ne 'ARRAY' ) {
        $analysis_html = '<h2>'.
            &mt('There is no submission data for this resource').
            '</h2>';
        $r->print($analysis_html);
        return;
    }
    #
    # This next call causes all the waiting around that people complain about
    my ($max,$min) = 
        &Apache::lonstathelpers::GetStudentAnswers($r,$problem,$Students,
                                                   'Statistics',
                                                   'stats_status');
    return if ($c->aborted());
    #
    # Collate the data
    my %Data;
    foreach my $student (@$Students) {
        my $answer = $student->{'answer'};
        $Data{$answer}++;
    }
    my @Labels = sort {$a <=> $b } keys(%Data);
    my @PlotData = @Data{@Labels};
    #
    my $width  = 500; 
    my $height = 100;
    my $plot = &one_dimensional_plot($r,500,100,scalar(@$Students),
                                     \@Labels,\@PlotData);
    $r->print($plot);
    return;
}

sub one_dimensional_plot {
    my ($r,$width,$height,$N,$Labels,$Data)=@_;
    #
    # Compute data -> image scaling factors
    my $min = $Labels->[0];
    my $max = $Labels->[-1];
    if ($max == $min) {
        $max =$min+1;
    }
    my $h_scale = ($width-10)/($max-$min);
    #
    my $max_y = 0;
    foreach (@$Data) {
        $max_y = $_ if ($max_y < $_);
    }
    my $ticscale = 5;
    if ($max_y * $ticscale > $height/2) {
        $ticscale = int($height/2/$max_y);
        $ticscale = 1 if ($ticscale < 1);
    }
    #
    # Create the plot
    my $plot = 
        qq{<drawimage width="$width" height="$height" bgcolor="transparent" >};
    for (my $idx=0;$idx<scalar(@$Labels);$idx++) {
        my $xloc = 5+$h_scale*($Labels->[$idx] - $min);
        my $top    = $height/2-$Data->[$idx]*$ticscale;
        my $bottom = $height/2+$Data->[$idx]*$ticscale;
        $plot .= 
            &line($xloc,$top,$xloc,$bottom,'888888',1);
    }
    #
    # Put the scale on last to ensure it is on top of the data.
    if ($min < 0 && $max > 0) {
        my $circle_x = 5+$h_scale*abs($min);  # '0' in data coordinates
        my $r = 4;
        $plot .= &line(5,$height/2,$circle_x-$r,$height/2,'000000',1);
        $plot .= &circle($circle_x,$height/2,$r+1,'000000');
        $plot .= &line($circle_x+$r,$height/2,$width-5,$height/2,'000000',1);
    } else {
        $plot .= &line(5,$height/2,$width-5,$height/2,'000000',1);
    }
    $plot .= '</drawimage>';
    my $plotresult =  &Apache::lonxml::xmlparse($r,'web',$plot);
    
    my $title = 'Distribution of correct answers';
    my $result = '<table>'.
        '<tr><td colspan="3" align="center">'.
        '<font size="+2">'.$title.' (N='.$N.')'.
        '</font>'.
        '</td></tr>'.
        '<tr>'.
        '<td valign="center">'.$min.'</td>'.
        '<td>'.$plotresult.'</td>'.
        '<td valign="center">'.$max.'</td>'.
        '</tr>'.
        '<tr><td colspan="3" align="center">'.
        'Maximum Number of Coinciding Values: '.$max_y.
        '</td></tr>'.
        '</table>';
    return $result;
}

##
## Helper subroutines for <drawimage>.  
## These should probably go somewhere more suitable soon.
sub line {
    my ($x1,$y1,$x2,$y2,$color,$thickness) = @_;
    return qq{<line x1="$x1" y1="$y1" x2="$x2" y2="$y2" color="$color" thickness="$thickness" />$/};
}

sub text {
    my ($x,$y,$color,$text,$font,$direction) = @_;
    if (! defined($font) || $font !~ /^(tiny|small|medium|large|giant)$/) {
        $font = 'medium';
    }
    if (! defined($direction) || $direction ne 'vertical') {
        $direction = '';
    }
    return qq{<text x="$x" y="$y" color="$color" font="$font" direction="$direction" >$text</text>};
}

sub rectangle {
    my ($x1,$y1,$x2,$y2,$color,$thickness,$filled) = @_;
    return qq{<rectangle x1="$x1" y1="$y1" x2="$x2" y2="$y2" color="$color" thickness="$thickness" filled="$filled" />};
}

sub arc {
    my ($x,$y,$width,$height,$start,$end,$color,$thickness,$filled)=@_;
    return qq{<arc x="$x" y="$y" width="$width" height="$height" start="$start" end="$end" color="$color" thickness="$thickness" filled="$filled" />};
}

sub circle {
    my ($x,$y,$radius,$color,$thickness,$filled)=@_;
    return &arc($x,$y,$radius,$radius,0,360,$color,$thickness,$filled);
}

sub build_student_data_worksheet {
    my ($workbook,$format) = @_;
    my $rows_output = 3;
    my $cols_output = 0;
    my $worksheet  = $workbook->addworksheet('Student Data');
    $worksheet->write($rows_output++,0,'Student Data',$format->{'h3'});
    my @Headers = ('full name','username','domain','section',
                   "student\nnumber",'identifier');
    $worksheet->write_row($rows_output++,0,\@Headers,$format->{'header'});
    my @Students = @Apache::lonstatistics::Students;
    my $studentrows = &Apache::loncoursedata::get_student_data(\@Students);
    my %ids;
    foreach my $row (@$studentrows) {
        my ($mysqlid,$student) = @$row;
        $ids{$student}=$mysqlid;
    }
    foreach my $student (@Students) {
        my $name_domain = $student->{'username'}.':'.$student->{'domain'};
        $worksheet->write_row($rows_output++,0,
                          [$student->{'fullname'},
                           $student->{'username'},$student->{'domain'},
                           $student->{'section'},$student->{'id'},
                           $ids{$name_domain}]);
    }
    return $worksheet;
}

#########################################################
#########################################################
##
##      Radio Response Routines
##
#########################################################
#########################################################
sub RadioResponseAnalysis {
    my ($r,$problem,$ProblemData,$Students) = @_;
    my ($resource,$respid) = ($problem->{'resource'},
                              $problem->{'respid'});
    my $analysis_html;
    my $PerformanceData = &Apache::loncoursedata::get_response_data
        (\@Apache::lonstatistics::SelectedSections,
         $Apache::lonstatistics::enrollment_status,
         $resource->{'symb'},$respid);
    if (! defined($PerformanceData) || 
        ref($PerformanceData) ne 'ARRAY' ) {
        $analysis_html = '<h2>'.
            &mt('There is no submission data for this resource').
            '</h2>';
        $r->print($analysis_html);
        return;
    }
    if (exists($ENV{'form.ExcelOutput'})) {
        $analysis_html .= &RR_Excel_output($r,$problem->{'resource'},
                                           $PerformanceData,$ProblemData);
    } elsif ($ENV{'form.AnalyzeOver'} eq 'Tries') {
        $analysis_html .= &RR_Tries_Analysis($r,$problem->{'resource'},
                                             $PerformanceData,$ProblemData);
    } elsif ($ENV{'form.AnalyzeOver'} eq 'Time') {
        $analysis_html .= &RR_Time_Analysis($r,$problem->{'resource'},
                                            $PerformanceData,$ProblemData);
    } else {
        $analysis_html .= '<h2>'.
           &mt('The analysis you have selected is not supported at this time').
           '</h2>';
    }
    $r->print($analysis_html);
}

sub RR_Excel_output   { 
    my ($r,$PerformanceData,$ProblemData) = @_;
    return '<h1>No!</h1>';
}

sub RR_Tries_Analysis { 
    my ($r,$resource,$PerformanceData,$ProblemData) = @_;
    my $analysis_html;
    my $mintries = 1;
    my $maxtries = $ENV{'form.NumPlots'};
    my ($table,$Foils,$Concepts) = &build_foil_index($ProblemData);
    if ((! defined($Concepts)) || ((@$Concepts < 2) && 
                                   ($ENV{'form.AnalyzeAs'} ne 'Foils'))) {
        $table = '<h3>'.
            &mt('Not enough data for concept analysis.  '.
                'Performing Foil Analysis').
            '</h3>'.$table;
        $ENV{'form.AnalyzeAs'} = 'Foils';
    }
    $analysis_html .= $table;
    my @TryData = &RR_tries_data_analysis($r,$PerformanceData);
#    if ($ENV{'form.AnalyzeAs'} eq 'Foils') {
        $analysis_html .= &RR_Tries_Foil_Analysis($mintries,$maxtries,$Foils,
                                                 \@TryData,$ProblemData);
#    } else {
#        $analysis_html = &RR_Tries_Concept_Analysis($mintries,$maxtries,
#                                                    $Concepts,
#                                                    \@TryData,
#                                                    $ProblemData);
#    }
    return $analysis_html;
}

sub RR_tries_data_analysis {
    my ($r,$Attempt_data) = @_;
    my @TryData;
    foreach my $attempt (@$Attempt_data) {
        my %Attempt = &hashify_attempt($attempt);
        my ($answer,undef) = split('=',$Attempt{'submission'});
        $TryData[$Attempt{'tries'}]->{$answer}++;
    }
    return @TryData;
}

sub RR_Time_Analysis  { 
    my ($r,$PerformanceData,$ProblemData) = @_;
    my $html;
    return $html;
}

sub RR_Tries_Foil_Analysis {
    my ($min,$max,$Foils,$TryData,$ProblemData) = @_;
    my $html;
    #
    # Compute the data neccessary to make the plots
    for (my $try=$min;$try<=$max;$try++) {
        my @PlotData_Correct; 
        my @PlotData_Incorrect;
        next if ($try > scalar(@{$TryData}));
        next if (! defined($TryData->[$try]));
        my %DataSet = %{$TryData->[$try]};
        my $total = 0;
        foreach my $foilid (@$Foils) {
            $total += $DataSet{$foilid};
        }
        foreach my $foilid (@$Foils) {
            if ($total == 0) {
                push (@PlotData_Correct,0);
                push (@PlotData_Incorrect,0);
            } else {
                if ($ProblemData->{'_Foils'}->{$foilid}->{'value'} eq 'true') {
                    push (@PlotData_Correct,
                          int(100*$DataSet{$foilid}/$total));
                    push (@PlotData_Incorrect,0);
                } else {
                    push (@PlotData_Correct,0);
                    push (@PlotData_Incorrect,
                          int(100*$DataSet{$foilid}/$total));
                }
            }
        }
        my $title='Attempt '.$try;
        my $xlabel = $total.' Submissions';
        $html.=  &Apache::loncommon::DrawBarGraph($title,
                                                  $xlabel,
                                                  'Percent Choosing',
                                                  100,
                                                  ['#33ff00','#ff3300'],
                                                  undef,
                                                  \@PlotData_Correct,
                                                  \@PlotData_Incorrect);
    }
    return $html;
}

sub RR_Tries_Concept_Analysis {
    my ($min,$max,$Concepts,$ResponseData,$ProblemData) = @_;
    my $html;
    return $html;
}

sub RR_Time_Foil_Analysis {
    my ($min,$max,$Foils,$ResponseData,$ProblemData) = @_;
    my $html;
    return $html;
}

sub RR_Time_Concept_Analysis {
    my ($min,$max,$Concepts,$ResponseData,$ProblemData) = @_;
    my $html;
    return $html;
}


sub get_Radio_problem_data {
    my ($url) = @_;
    my $Answ=&Apache::lonnet::ssi($url,('grade_target' => 'analyze'));
    (my $garbage,$Answ)=split('_HASH_REF__',$Answ,2);
    my %Answer = &Apache::lonnet::str2hash($Answ);
    my %Partdata;
    foreach my $part (@{$Answer{'parts'}}) {
        while (my($key,$value) = each(%Answer)) {
#            if (ref($value) eq 'ARRAY') {
#                &Apache::lonnet::logthis('is ref part:'.$part.' '.$key.'='.join(',',@$value));
#            } else {
#                &Apache::lonnet::logthis('notref part:'.$part.' '.$key.'='.$value);
#            }                
            next if ($key !~ /^$part/);
            $key =~ s/^$part\.//;
            if ($key eq 'foils') {
                $Partdata{$part}->{'_Foils'}=$value;
            } elsif ($key eq 'options') {
                $Partdata{$part}->{'_Options'}=$value;
            } elsif ($key eq 'shown') {
                $Partdata{$part}->{'_Shown'}=$value;
            } elsif ($key =~ /^foil.value.(.*)$/) {
                $Partdata{$part}->{$1}->{'value'}=$value;
            } elsif ($key =~ /^foil.text.(.*)$/) {
                $Partdata{$part}->{$1}->{'text'}=$value;
            }
        }
    }
    return %Partdata;
}

#########################################################
#########################################################
##
##      Option Response Routines
##
#########################################################
#########################################################
sub OptionResponseAnalysis {
    my ($r,$problem,$ProblemData,$Students) = @_;
    my ($resource,$respid) = ($problem->{'resource'},
                              $problem->{'respid'});
    # Note: part data is not needed.
    my $PerformanceData = &Apache::loncoursedata::get_response_data
        (\@Apache::lonstatistics::SelectedSections,
         $Apache::lonstatistics::enrollment_status,
         $resource->{'symb'},$respid);
    if (! defined($PerformanceData) || 
        ref($PerformanceData) ne 'ARRAY' ) {
        $r->print('<h2>'.
                  &mt('There is no student data for this problem.').
                  '</h2>');
    }  else {
        $r->rflush();
        if (exists($ENV{'form.ExcelOutput'})) {
            my $result = &OR_excel_sheet($r,$resource,
                                         $PerformanceData,
                                         $ProblemData);
            $r->print($result);
            $r->rflush();
        } else {
            if ($ENV{'form.AnalyzeOver'} eq 'Tries') {
                my $analysis_html = &OR_tries_analysis($r,
                                                    $PerformanceData,
                                                    $ProblemData);
                $r->print($analysis_html);
                $r->rflush();
            } elsif ($ENV{'form.AnalyzeOver'} eq 'Time') {
                my $analysis_html = &OR_time_analysis($PerformanceData,
                                                   $ProblemData);
                $r->print($analysis_html);
                $r->rflush();
            } else {
                $r->print('<h2>'.
                          &mt('The analysis you have selected is '.
                              'not supported at this time').
                          '</h2>');
            }
        }
    }
}

#########################################################
#
#       Option Response:  Tries Analysis
#
#########################################################
sub OR_tries_analysis {
    my ($r,$PerformanceData,$ORdata) = @_;
    my $mintries = 1;
    my $maxtries = $ENV{'form.NumPlots'};
    my ($table,$Foils,$Concepts) = &build_foil_index($ORdata);
    if (! defined($Concepts) || 
        ((@$Concepts < 2) && ($ENV{'form.AnalyzeAs'} ne 'Foils'))) {
        $table = '<h3>'.
            &mt('Not enough data for concept analysis.  '.
                'Performing Foil Analysis').
            '</h3>'.$table;
        $ENV{'form.AnalyzeAs'} = 'Foils';
    }
    my %ResponseData = &OR_analyze_by_tries($r,$PerformanceData,
                                                     $mintries,$maxtries);
    my $analysis = '';
    if ($ENV{'form.AnalyzeAs'} eq 'Foils') {
        $analysis = &OR_Tries_Foil_Analysis($mintries,$maxtries,$Concepts,
                                         \%ResponseData,$ORdata);
    } else {
        $analysis = &OR_Tries_Concept_Analysis($mintries,$maxtries,
                                            $Concepts,\%ResponseData,$ORdata);
    }
    $table .= $analysis;
    return $table;
}

sub OR_Tries_Foil_Analysis {
    my ($mintries,$maxtries,$Concepts,$respdat,$ORdata) = @_;
    my %ResponseData = %$respdat;
    #
    # Compute the data neccessary to make the plots
    my @PlotData; 
    foreach my $concept (@$Concepts) {
        foreach my $foilid (@{$concept->{'foils'}}) {
            for (my $try=$mintries;$try<=$maxtries;$try++) {
                if ($ResponseData{$foilid}->[$try]->{'_total'} == 0) {
                    push(@{$PlotData[$try]->{'_correct'}},0);
                } else {
                    push(@{$PlotData[$try]->{'_correct'}},
                         100*$ResponseData{$foilid}->[$try]->{'_correct'}/
                         $ResponseData{$foilid}->[$try]->{'_total'});
                }
                foreach my $option (@{$ORdata->{'_Options'}}) {
                    push(@{$PlotData[$try]->{'_total'}},
                         $ResponseData{$foilid}->[$try]->{'_total'});
                    if ($ResponseData{$foilid}->[$try]->{'_total'} == 0) {
                        push (@{$PlotData[$try]->{$option}},0);
                    } else {
                        if ($ResponseData{$foilid}->[$try]->{'_total'} ==
                            $ResponseData{$foilid}->[$try]->{'_correct'}) {
                            push(@{$PlotData[$try]->{$option}},0);
                        } else {
                            push (@{$PlotData[$try]->{$option}},
                                  100 * 
                                  $ResponseData{$foilid}->[$try]->{$option} / 
                                  ($ResponseData{$foilid}->[$try]->{'_total'} 
                                   - 
                                   $ResponseData{$foilid}->[$try]->{'_correct'}
                                   ));
                        }
                    }
                } # End of foreach my $option
            }
        } # End of foreach my $foilid
    } # End of foreach my $concept
    # 
    # Build a table for the plots
    my $analysis_html = "<table>\n";
    my $optionkey = &build_option_index($ORdata);
    for (my $try=$mintries;$try<=$maxtries;$try++) {
        my $count = $ResponseData{'_total'}->[$try];
        my $title = 'Submission '.$try.' (N='.$count.')';
        my @Datasets;
        foreach my $option ('_correct',@{$ORdata->{'_Options'}}) {
            next if (! exists($PlotData[$try]->{$option}));
            push(@Datasets,$PlotData[$try]->{$option});
        }
        #
        # Put a blank in the data set between concepts
        for (my $set =0;$set<=$#Datasets;$set++) {
            my @Data = @{$Datasets[$set]};
            my $idx = 0;
            foreach my $concept (@{$Concepts}) {
                foreach my $foilid (@{$concept->{'foils'}}) {
                    $Datasets[$set]->[$idx++]=shift(@Data);
                }
                if ($concept->{'name'} ne $Concepts->[-1]->{'name'}) {
                    $Datasets[$set]->[$idx++] = 0;
                }
            }
        }
        #
        # Set up the labels needed for the bar graph
        my @Labels;
        my $idx = 1;
        foreach my $concept (@{$Concepts}) {
            foreach my $foilid (@{$concept->{'foils'}}) {
                push(@Labels,$idx++);
            }
            push(@Labels,'');
        }
        #
        my $correctgraph = &Apache::loncommon::DrawBarGraph
            ($title,'Foil Number','Percent Correct',
             100,$plotcolors,\@Labels,$Datasets[0]);
        $analysis_html.= '<tr><td>'.$correctgraph.'</td>';
        
        #
        #
        next if (! defined($Datasets[0]));
        for (my $i=0; $i< scalar(@{$Datasets[0]});$i++) {
            $Datasets[0]->[$i]=0;
        }
        $count = $ResponseData{'_total'}->[$try] - 
                                           $ResponseData{'_correct'}->[$try];
        $title = 'Submission '.$try.' (N='.$count.')';
        my $incorrectgraph = &Apache::loncommon::DrawBarGraph
            ($title,'Foil Number','% Option Chosen Incorrectly',
             100,$plotcolors,\@Labels,@Datasets);
        $analysis_html.= '<td>'.$incorrectgraph.'</td>';
        $analysis_html.= '<td>'.$optionkey."<td></tr>\n";
    }
    $analysis_html .= "</table>\n";
    return $analysis_html;
}

sub OR_Tries_Concept_Analysis {
    my ($mintries,$maxtries,$Concepts,$respdat,$ORdata) = @_;
    my %ResponseData = %$respdat;
    my $analysis_html = "<table>\n";
    #
    # Compute the data neccessary to make the plots
    my @PlotData;
    # Concept analysis
    #
    # Note: we do not bother with characterizing the students incorrect
    # answers at the concept level because an incorrect answer for one foil
    # may be a correct answer for another foil.
    my %ConceptData;
    foreach my $concept (@{$Concepts}) {
        for (my $i=$mintries;$i<=$maxtries;$i++) {
            #
            # Gather the per-attempt data
            my $cdata = $ConceptData{$concept}->[$i];
            foreach my $foilid (@{$concept->{'foils'}}) {
                $cdata->{'_correct'} += 
                    $ResponseData{$foilid}->[$i]->{'_correct'};
                $cdata->{'_total'}   += 
                    $ResponseData{$foilid}->[$i]->{'_total'};
            }
            push (@{$PlotData[$i]->{'_total'}},$cdata->{'_total'});
            if ($cdata->{'_total'} == 0) {
                push (@{$PlotData[$i]->{'_correct'}},0);
            } else {
                push (@{$PlotData[$i]->{'_correct'}},
                      100*$cdata->{'_correct'}/$cdata->{'_total'});
                }
        }
    }
    # Build a table for the plots
    for (my $i=$mintries;$i<=$maxtries;$i++) {
        my $minstu = $PlotData[$i]->{'_total'}->[0];
        my $maxstu = $PlotData[$i]->{'_total'}->[0];
        foreach my $count (@{$PlotData[$i]->{'_total'}}) {
            if ($minstu > $count) {
                $minstu = $count;
            }
            if ($maxstu < $count) {
                $maxstu = $count;
            }
        }
        $maxstu = 0 if (! defined($maxstu));
        $minstu = 0 if (! defined($minstu));
        my $title;
        my $count = $ResponseData{'_total'}->[$i];
        if ($count == 0) {
            $count = 'no submissions';
        } elsif ($count == 1) {
            $count = '1 submission';
        } else {
            $count = $count.' submissions';
        }
        $title = 'Attempt '.$i.', '.$count;
        my $graphlink = &Apache::loncommon::DrawBarGraph
            ($title,'Concept Number','Percent Correct',
             100,$plotcolors,undef,$PlotData[$i]->{'_correct'});
        $analysis_html.= '<tr><td>'.$graphlink."</td></tr>\n";
    }
    $analysis_html .= "</table>\n";
    return $analysis_html;
}

sub OR_analyze_by_tries {
    my ($r,$PerformanceData,$mintries,$maxtries) = @_;
    my %Trydata;
    $mintries = 1         if (! defined($mintries) || $mintries < 1);
    $maxtries = $mintries if (! defined($maxtries) || $maxtries < $mintries);
    foreach my $row (@$PerformanceData) {
        next if (! defined($row));
        my $tries = &get_tries_from_row($row);
        my %Row   = &Process_OR_Row($row);
        next if (! %Row);
        while (my ($foilid,$href) = each(%Row)) {
            if (! ref($href)) { 
                $Trydata{$foilid}->[$tries] += $href;
                next;
            }
            while (my ($option,$value) = each(%$href)) {
                $Trydata{$foilid}->[$tries]->{$option}+=$value;
            }
        }
    }
    return %Trydata;
}

#########################################################
#
#     Option Response: Time Analysis
#
#########################################################
sub OR_time_analysis {
    my ($PerformanceData,$ORdata) = @_;
    my ($table,$Foils,$Concepts) = &build_foil_index($ORdata);
    if ((@$Concepts < 2) && ($ENV{'form.AnalyzeAs'} ne 'Foils')) {
        $table = '<h3>'.
            &mt('Not enough data for concept analysis.  '.
                'Performing Foil Analysis').
            '</h3>'.$table;
        $ENV{'form.AnalyzeAs'} = 'Foils';
    }
    my $num_plots = $ENV{'form.NumPlots'};
    my $num_data = scalar(@$PerformanceData)-1;
    my $percent = sprintf('%2f',100/$num_plots);
    #
    $table .= "<table>\n";
    for (my $i=0;$i<$num_plots;$i++) {
        ##
        my $starttime = &Apache::lonhtmlcommon::get_date_from_form
            ('startdate_'.$i);
        my $endtime = &Apache::lonhtmlcommon::get_date_from_form
            ('enddate_'.$i);
        if (! defined($starttime) || ! defined($endtime)) {
            my $sec_in_day = 86400;
            my $last_sub_time = &get_time_from_row($PerformanceData->[-1]);
            my ($sday,$smon,$syear);
            (undef,undef,undef,$sday,$smon,$syear) = 
                localtime($last_sub_time - $sec_in_day*$i);
            $starttime = &Time::Local::timelocal(0,0,0,$sday,$smon,$syear);
            $endtime = $starttime + $sec_in_day;
            if ($i == ($num_plots -1 )) {
                $starttime = &get_time_from_row($PerformanceData->[0]);
            }
        }
        my $startdateform = &Apache::lonhtmlcommon::date_setter
            ('Statistics','startdate_'.$i,$starttime);
        my $enddateform = &Apache::lonhtmlcommon::date_setter
            ('Statistics','enddate_'.$i,$endtime);
        #
        my $begin_index;
        my $end_index;
        my $j;
        while (++$j < scalar(@$PerformanceData)) {
            last if (&get_time_from_row($PerformanceData->[$j]) 
                                                              > $starttime);
        }
        $begin_index = $j;
        while (++$j < scalar(@$PerformanceData)) {
            last if (&get_time_from_row($PerformanceData->[$j]) > $endtime);
        }
        $end_index = $j;
        ##
        my $interval = {
            begin_index => $begin_index,
            end_index   => $end_index,
            startdateform => $startdateform,
            enddateform   => $enddateform,
        };
        if ($ENV{'form.AnalyzeAs'} eq 'Foils') {
            $table .= &OR_Foil_Time_Analysis($PerformanceData,$ORdata,$Foils,
                                          $interval);
        } else {
            $table .= &OR_Concept_Time_Analysis($PerformanceData,$ORdata,
                                             $Concepts,$interval);
        }
    }
    #
    return $table;
}

sub OR_Foil_Time_Analysis {
    my ($PerformanceData,$ORdata,$Foils,$interval) = @_;
    my $analysis_html;
    my $foilkey = &build_option_index($ORdata);
    my ($begin_index,$end_index) = ($interval->{'begin_index'},
                                    $interval->{'end_index'});
    my %TimeData;
    #
    # Compute the number getting the foils correct or incorrects
    for (my $j=$begin_index;$j<=$end_index;$j++) {
        my $row = $PerformanceData->[$j];
        next if (! defined($row));
        my %Row = &Process_OR_Row($row);
        while (my ($foilid,$href) = each(%Row)) {
            if (! ref($href)) {
                $TimeData{$foilid} += $href;
                next;
            }
            while (my ($option,$value) = each(%$href)) {
                $TimeData{$foilid}->{$option}+=$value;
            }
        }
    }
    my @Plotdata;
    foreach my $foil (@$Foils) {
        my $total = $TimeData{$foil}->{'_total'};
        if ($total == 0) {
            push(@{$Plotdata[0]},0);
        } else {
            push(@{$Plotdata[0]},
                 100 * $TimeData{$foil}->{'_correct'} / $total);
        }
        my $total_incorrect = $total - $TimeData{$foil}->{'_correct'};
        my $optionidx = 1;
        foreach my $option (@{$ORdata->{'_Options'}}) {
            if ($total_incorrect == 0) {
                push(@{$Plotdata[$optionidx]},0);
            } else {
                push(@{$Plotdata[$optionidx]},
                     100 * $TimeData{$foil}->{$option} / $total_incorrect);
            }
        } continue {
            $optionidx++;
        }
    }
    #
    # Create the plot
    my $count = $end_index-$begin_index;
    my $title;
    if ($count == 0) {
        $title = 'no submissions';
    } elsif ($count == 1) {
        $title = 'one submission';
    } else {
        $title = $count.' submissions';
    }
    my $correctplot = &Apache::loncommon::DrawBarGraph($title,
                                                       'Foil Number',
                                                       'Percent Correct',
                                                       100,
                                                       $plotcolors,
                                                       undef,
                                                       $Plotdata[0]);
    for (my $j=0; $j< scalar(@{$Plotdata[0]});$j++) {
        $Plotdata[0]->[$j]=0;
    }
    $count = $end_index-$begin_index-$TimeData{'_correct'};
    if ($count == 0) {
        $title = 'no submissions';
    } elsif ($count == 1) {
        $title = 'one submission';
    } else {
        $title = $count.' submissions';
    }
    my $incorrectplot = &Apache::loncommon::DrawBarGraph($title,
                                                 'Foil Number',
                                                 'Incorrect Option Choice',
                                                 100,
                                                 $plotcolors,
                                                 undef,
                                                 @Plotdata);        
    $analysis_html.='<tr>'.
        '<td>'.$correctplot.'</td>'.
        '<td>'.$incorrectplot.'</td>'.
        '<td align="left" valign="top">'.$foilkey.'</td>'."</tr>\n";
    $analysis_html.= '<tr>'.'<td colspan="3">'.
        '<b>Start Time</b>:'.
        ' &nbsp;'.$interval->{'startdateform'}.'<br />'.
        '<b>End Time</b>&nbsp;&nbsp;: '.
        '&nbsp;'.$interval->{'enddateform'}.'<br />'.
#        '<b>Plot Title</b>&nbsp;&nbsp;:'.
#        ("&nbsp;"x3).$interval->{'titleform'}.
        '</td>'.
        "</tr>\n";
    return $analysis_html;
}

sub OR_Concept_Time_Analysis {
    my ($PerformanceData,$ORdata,$Concepts,$interval) = @_;
    my $analysis_html;
    ##
    ## Determine starttime, endtime, startindex, endindex
    my ($begin_index,$end_index) = ($interval->{'begin_index'},
                                    $interval->{'end_index'});
    my %TimeData;
    #
    # Compute the number getting the foils correct or incorrects
    for (my $j=$begin_index;$j<=$end_index;$j++) {
        my $row = $PerformanceData->[$j];
        next if (! defined($row));
        my %Row = &Process_OR_Row($row);
        while (my ($foilid,$href) = each(%Row)) {
            if (! ref($href)) {
                $TimeData{$foilid} += $href;
                next;
            }
            while (my ($option,$value) = each(%$href)) {
                $TimeData{$foilid}->{$option}+=$value;
            }
        }
    }
    #
    # Put the data in plottable form
    my @Plotdata;
    foreach my $concept (@$Concepts) {
        my ($total,$correct);
        foreach my $foil (@{$concept->{'foils'}}) {
            $total += $TimeData{$foil}->{'_total'};
            $correct += $TimeData{$foil}->{'_correct'};
        }
        if ($total == 0) {
            push(@Plotdata,0);
        } else {
            push(@Plotdata,100 * $correct / $total);
        }
    }
    #
    # Create the plot
    my $title = ($end_index - $begin_index).' submissions';
    my $correctplot = &Apache::loncommon::DrawBarGraph($title,
                                                    'Concept Number',
                                                    'Percent Correct',
                                                    100,
                                                    $plotcolors,
                                                    undef,
                                                    \@Plotdata);
    $analysis_html.='<tr>'.
        '<td>'.$correctplot.'</td>'.
        '<td align="left" valign="top">'.
        '<b>Start Time</b>: &nbsp;'.$interval->{'startdateform'}.'<br />'.
        '<b>End Time</b>&nbsp;&nbsp;: '.
           '&nbsp;'.$interval->{'enddateform'}.'<br />'.
#        '<b>Plot Title</b>&nbsp;&nbsp;:'.("&nbsp;"x3).
#            $interval->{'titleform'}.
        '</td>'.
        "</tr>\n";
    return $analysis_html;
}

#########################################################
#########################################################
##
##             Excel output 
##
#########################################################
#########################################################
sub OR_excel_sheet {
    my ($r,$resource,$PerformanceData,$ORdata) = @_;
    my $response = '';
    my (undef,$Foils,$Concepts) = &build_foil_index($ORdata);
    #
    # Create excel worksheet
    my $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
        time.'_'.rand(1000000000).'.xls';
    my $workbook  = Spreadsheet::WriteExcel->new('/home/httpd'.$filename);
    if (! defined($workbook)) {
        $r->log_error("Error creating excel spreadsheet $filename: $!");
        $r->print('<p>'.&mt("Unable to create new Excel file.  ".
                            "This error has been logged.  ".
                            "Please alert your LON-CAPA administrator").
                  '</p>');
        return undef;
    }
    #
    $workbook->set_tempdir('/home/httpd/perl/tmp');
    my $format = &Apache::loncommon::define_excel_formats($workbook);
    #
    # Create and populate main worksheets
    my $problem_data_sheet  = $workbook->addworksheet('Problem Data');
    my $student_data_sheet = &build_student_data_worksheet($workbook,$format);
    my $response_data_sheet = $workbook->addworksheet('Response Data');
    foreach my $sheet ($problem_data_sheet,$student_data_sheet,
                       $response_data_sheet) {
        $sheet->write(0,0,$resource->{'title'},$format->{'h2'});
        $sheet->write(1,0,$resource->{'src'},$format->{'h3'});
    }
    #
    my $result;
    $result = &OR_build_problem_data_worksheet($problem_data_sheet,$format,
                                            $Concepts,$ORdata);
    if ($result ne 'okay') {
        # Do something useful
    }
    $result = &OR_build_response_data_worksheet($response_data_sheet,$format,
                                             $PerformanceData,$Foils,
                                             $ORdata);
    if ($result ne 'okay') {
        # Do something useful
    }
    $response_data_sheet->activate();
    #
    # Close the excel file
    $workbook->close();
    #
    # Write a link to allow them to download it
    $result .= '<h2>'.&mt('Excel Raw Data Output').'</h2>'.
              '<p><a href="'.$filename.'">'.
              &mt('Your Excel spreadsheet.').
              '</a></p>'."\n";
    return $result;
}

sub OR_build_problem_data_worksheet {
    my ($worksheet,$format,$Concepts,$ORdata) = @_;
    my $rows_output = 3;
    my $cols_output = 0;
    $worksheet->write($rows_output++,0,'Problem Structure',$format->{'h3'});
    ##
    ##
    my @Headers;
    if (@$Concepts > 1) {
        @Headers = ("Concept\nNumber",'Concept',"Foil\nNumber",
                    'Foil Name','Foil Text','Correct value');
    } else {
        @Headers = ('Foil Number','FoilName','Foil Text','Correct value');
    }
    $worksheet->write_row($rows_output++,0,\@Headers,$format->{'header'});
    my %Foildata = %{$ORdata->{'_Foils'}};
    my $conceptindex = 1;
    my $foilindex = 1;
    foreach my $concept (@$Concepts) {
        my @FoilsInConcept = @{$concept->{'foils'}};
        my $firstfoil = shift(@FoilsInConcept);
        if (@$Concepts > 1) {
            $worksheet->write_row($rows_output++,0,
                                  [$conceptindex,
                                   $concept->{'name'},
                                   $foilindex++,
                                   $Foildata{$firstfoil}->{'name'},
                                   $Foildata{$firstfoil}->{'text'},
                                   $Foildata{$firstfoil}->{'value'},]);
        } else {
            $worksheet->write_row($rows_output++,0,
                                  [ $foilindex++,
                                    $Foildata{$firstfoil}->{'name'},
                                    $Foildata{$firstfoil}->{'text'},
                                    $Foildata{$firstfoil}->{'value'},]);
        }
        foreach my $foilid (@FoilsInConcept) {
            if (@$Concepts > 1) {
                $worksheet->write_row($rows_output++,0,
                                      ['',
                                       '',
                                       $foilindex,
                                       $Foildata{$foilid}->{'name'},
                                       $Foildata{$foilid}->{'text'},
                                       $Foildata{$foilid}->{'value'},]);
            } else {
                $worksheet->write_row($rows_output++,0,                
                                      [$foilindex,
                                       $Foildata{$foilid}->{'name'},
                                       $Foildata{$foilid}->{'text'},
                                       $Foildata{$foilid}->{'value'},]);
            }                
        } continue {
            $foilindex++;
        }
    } continue {
        $conceptindex++;
    }
    $rows_output++;
    $rows_output++;
    ##
    ## Option data output
    $worksheet->write($rows_output++,0,'Options',$format->{'header'});
    foreach my $string (@{$ORdata->{'_Options'}}) {
        $worksheet->write($rows_output++,0,$string);
    }
    return 'okay';
}

sub OR_build_response_data_worksheet {
    my ($worksheet,$format,$PerformanceData,$Foils,$ORdata)=@_;
    my $rows_output = 3;
    my $cols_output = 0;
    $worksheet->write($rows_output++,0,'Response Data',$format->{'h3'});
    $worksheet->set_column(1,1,20);
    $worksheet->set_column(2,2,13);
    my @Headers = ('identifier','time','award detail','attempt');
    foreach my $foil (@$Foils) {
        push (@Headers,$foil.' submission');
        push (@Headers,$foil.' grading');
    }
    $worksheet->write_row($rows_output++,0,\@Headers,$format->{'header'});
    #
    foreach my $row (@$PerformanceData) {
        next if (! defined($row));
        my ($student,$award,$grading,$submission,$time,$tries) = @$row;
        my @Foilgrades = split('&',$grading);
        my @Foilsubs   = split('&',$submission);
        my %ResponseData;
        for (my $j=0;$j<=$#Foilgrades;$j++) {
            my ($foilid,$correct)  = split('=',$Foilgrades[$j]);
            my (undef,$submission) = split('=',$Foilsubs[$j]);
            $submission = &Apache::lonnet::unescape($submission);
            $ResponseData{$foilid.' submission'}=$submission;
            $ResponseData{$foilid.' award'}=$correct;
        }
        $worksheet->write($rows_output,$cols_output++,$student);
        $worksheet->write($rows_output,$cols_output++,
             &Apache::lonstathelpers::calc_serial($time),$format->{'date'});
        $worksheet->write($rows_output,$cols_output++,$award);
        $worksheet->write($rows_output,$cols_output++,$tries);
        foreach my $foilid (@$Foils) {
            $worksheet->write($rows_output,$cols_output++,
                              $ResponseData{$foilid.' submission'});
            $worksheet->write($rows_output,$cols_output++,
                              $ResponseData{$foilid.' award'});
        }
        $rows_output++;
        $cols_output = 0;
    }
    return;
}

sub build_foil_index {
    my ($ORdata) = @_;
    return if (! exists($ORdata->{'_Foils'}));
    my %Foildata = %{$ORdata->{'_Foils'}};
    my @Foils = sort(keys(%Foildata));
    my %Concepts;
    foreach my $foilid (@Foils) {
        push(@{$Concepts{$Foildata{$foilid}->{'_Concept'}}},
             $foilid);
    }
    undef(@Foils);
    # Having gathered the concept information in a hash, we now translate it
    # into an array because we need to be consistent about order.
    # Also put the foils in order, too.
    my $sortfunction = sub {
        my %Numbers = (one   => 1,
                       two   => 2,
                       three => 3,
                       four  => 4,
                       five  => 5,
                       six   => 6,
                       seven => 7,
                       eight => 8,
                       nine  => 9,
                       ten   => 10,);
        my $a1 = lc($a); 
        my $b1 = lc($b);
        if (exists($Numbers{$a1})) {
            $a1 = $Numbers{$a1};
        }
        if (exists($Numbers{$b1})) {
            $b1 = $Numbers{$b1};
        }
        if (($a1 =~/^\d+$/) && ($b1 =~/^\d+$/)) {
            return $a1 <=> $b1;
        } else {
            return $a1 cmp $b1;
        }
    };
    my @Concepts;
    foreach my $concept (sort $sortfunction (keys(%Concepts))) {
        if (! defined($Concepts{$concept})) {
            $Concepts{$concept}=[];
#            next;
        }
        push(@Concepts,{ name => $concept,
                        foils => [@{$Concepts{$concept}}]});
        push(@Foils,(@{$Concepts{$concept}}));
    }
    #
    # Build up the table of row labels.
    my $table = '<table border="1" >'."\n";
    if (@Concepts > 1) {
        $table .= '<tr>'.
            '<th>'.&mt('Concept Number').'</th>'.
            '<th>'.&mt('Concept').'</th>'.
            '<th>'.&mt('Foil Number').'</th>'.
            '<th>'.&mt('Foil Name').'</th>'.
            '<th>'.&mt('Foil Text').'</th>'.
            '<th>'.&mt('Correct Value').'</th>'.
            "</tr>\n";
    } else {
        $table .= '<tr>'.
            '<th>'.&mt('Foil Number').'</th>'.
            '<th>'.&mt('Foil Name').'</th>'.
            '<th>'.&mt('Foil Text').'</th>'.
            '<th>'.&mt('Correct Value').'</th>'.
            "</tr>\n";
    }        
    my $conceptindex = 1;
    my $foilindex = 1;
    foreach my $concept (@Concepts) {
        my @FoilsInConcept = @{$concept->{'foils'}};
        my $firstfoil = shift(@FoilsInConcept);
        if (@Concepts > 1) {
            $table .= '<tr>'.
                '<td>'.$conceptindex.'</td>'.
                '<td>'.&HTML::Entities::encode($concept->{'name'},'<>&"').'</td>'.
                '<td>'.$foilindex++.'</td>'.
                '<td>'.&HTML::Entities::encode($Foildata{$firstfoil}->{'name'},'<>&"').'</td>'.
                '<td>'.$Foildata{$firstfoil}->{'text'}.'</td>'.
                '<td>'.&HTML::Entities::encode($Foildata{$firstfoil}->{'value'},'<>&"').'</td>'.
                "</tr>\n";
        } else {
            $table .= '<tr>'.
                '<td>'.$foilindex++.'</td>'.
                '<td>'.&HTML::Entities::encode($Foildata{$firstfoil}->{'name'},'<>&"').'</td>'.
                '<td>'.$Foildata{$firstfoil}->{'text'}.'</td>'.
                '<td>'.&HTML::Entities::encode($Foildata{$firstfoil}->{'value'},'<>&"').'</td>'.
                "</tr>\n";
        }
        foreach my $foilid (@FoilsInConcept) {
            if (@Concepts > 1) {
                $table .= '<tr>'.
                    '<td></td>'.
                    '<td></td>'.
                    '<td>'.$foilindex.'</td>'.
                    '<td>'.&HTML::Entities::encode($Foildata{$foilid}->{'name'},'<>&"').'</td>'.
                    '<td>'.$Foildata{$foilid}->{'text'}.'</td>'.
                    '<td>'.&HTML::Entities::encode($Foildata{$foilid}->{'value'},'<>&"').'</td>'.
                    "</tr>\n";
            } else {
                $table .= '<tr>'.
                    '<td>'.$foilindex.'</td>'.
                    '<td>'.&HTML::Entities::encode($Foildata{$foilid}->{'name'},'<>&"').'</td>'.
                    '<td>'.$Foildata{$foilid}->{'text'}.'</td>'.
                    '<td>'.&HTML::Entities::encode($Foildata{$foilid}->{'value'},'<>&"').'</td>'.
                    "</tr>\n";
            }                
        } continue {
            $foilindex++;
        }
    } continue {
        $conceptindex++;
    }
    $table .= "</table>\n";
    #
    # Build option index with color stuff
    return ($table,\@Foils,\@Concepts);
}

sub build_option_index {
    my ($ORdata)= @_;
    my $table = "<table>\n";
    my $optionindex = 0;
    my @Rows;
    foreach my $option (&mt('correct option chosen'),@{$ORdata->{'_Options'}}) {
        push (@Rows,
              '<tr>'.
              '<td bgcolor="'.$plotcolors->[$optionindex++].'">'.
              ('&nbsp;'x4).'</td>'.
              '<td>'.&HTML::Entities::encode($option,'<>&"').'</td>'.
              "</tr>\n");
    }
    shift(@Rows); # Throw away 'correct option chosen' color
    $table .= join('',reverse(@Rows));
    $table .= "</table>\n";
}

#########################################################
#########################################################
##
##   Generic Interface Routines
##
#########################################################
#########################################################
sub CreateInterface {
    ##
    ## Environment variable initialization
    if (! exists$ENV{'form.AnalyzeOver'}) {
        $ENV{'form.AnalyzeOver'} = 'Tries';
    }
    ##
    ## Build the menu
    my $Str = '';
    $Str .= &Apache::lonhtmlcommon::breadcrumbs
        (undef,'Detailed Problem Analysis');
    $Str .= '<table cellspacing="5">'."\n";
    $Str .= '<tr>';
    $Str .= '<td align="center"><b>'.&mt('Sections').'</b></td>';
    $Str .= '<td align="center"><b>'.&mt('Enrollment Status').'</b></td>';
#    $Str .= '<td align="center"><b>'.&mt('Sequences and Folders').'</b></td>';
    $Str .= '<td align="center">&nbsp;</td>';
    $Str .= '</tr>'."\n";
    ##
    ## 
    $Str .= '<tr><td align="center">'."\n";
    $Str .= &Apache::lonstatistics::SectionSelect('Section','multiple',5);
    $Str .= '</td>';
    #
    $Str .= '<td align="center">';
    $Str .= &Apache::lonhtmlcommon::StatusOptions(undef,undef,5);
    $Str .= '</td>';
    #
#    $Str .= '<td align="center">';
    my $only_seq_with_assessments = sub { 
        my $s=shift;
        if ($s->{'num_assess'} < 1) { 
            return 0;
        } else { 
            return 1;
        }
    };
    &Apache::lonstatistics::MapSelect('Maps','multiple,all',5,
                                              $only_seq_with_assessments);
    ##
    ##
    $Str .= '<td>';
    { # These braces are here to organize the code, not scope it.
        {
            $Str .= '<nobr>'.&mt('Analyze Over ');
            $Str .= &Apache::loncommon::help_open_topic
                                                  ('Analysis_Analyze_Over');
            $Str .='<select name="AnalyzeOver" >';
            $Str .= '<option value="Tries" ';
            if (! exists($ENV{'form.AnalyzeOver'}) || 
                $ENV{'form.AnalyzeOver'} eq 'Tries'){
                # Default to Tries
                $Str .= ' selected ';
            }
            $Str .= '>'.&mt('Tries').'</option>';
            $Str .= '<option value="Time" ';
            $Str .= ' selected ' if ($ENV{'form.AnalyzeOver'} eq 'Time');
            $Str .= '>'.&mt('Time').'</option>';
            $Str .= '</select>';
            $Str .= '</nobr><br />';
        }
        {
            $Str .= '<nobr>'.&mt('Analyze as ');
            $Str .= &Apache::loncommon::help_open_topic
                                                  ('Analysis_Analyze_as');
            $Str .='<select name="AnalyzeAs" >';
            $Str .= '<option value="Concepts" ';
            if (! exists($ENV{'form.AnalyzeAs'}) || 
                $ENV{'form.AnalyzeAs'} eq 'Concepts'){
                # Default to Concepts
                $Str .= ' selected ';
            }
            $Str .= '>'.&mt('Concepts').'</option>';
            $Str .= '<option value="Foils" ';
            $Str .= ' selected ' if ($ENV{'form.AnalyzeAs'} eq 'Foils');
            $Str .= '>'.&mt('Foils').'</option>';
            $Str .= '</select></nobr><br />';
        }
        {
            $Str .= '<nobr>'.&mt('Number of Plots:');
            $Str .= &Apache::loncommon::help_open_topic
                                                  ('Analysis_num_plots');
            $Str .= '<select name="NumPlots">';
            if (! exists($ENV{'form.NumPlots'}) 
                || $ENV{'form.NumPlots'} < 1 
                || $ENV{'form.NumPlots'} > 20) {
                $ENV{'form.NumPlots'} = 5;
            }
            foreach my $i (1,2,3,4,5,6,7,8,10,15,20) {
                $Str .= '<option value="'.$i.'" ';
                if ($ENV{'form.NumPlots'} == $i) { $Str.=' selected '; }
                $Str .= '>'.$i.'</option>';
            }
            $Str .= '</select></nobr><br />';
        }
        {
            $Str .= '<nobr>'.&mt('Status: [_1]',
                                 '<input type="text" '.
                                 'name="stats_status" size="60" value="" />'
                                 ).
                    '</nobr><br />';
        }
    }
    $Str .= '</td>';
    ##
    ##
    $Str .= '</tr>'."\n";
    $Str .= '</table>'."\n";
    return $Str;
}

#########################################################
#########################################################
##
##              Misc Option Response functions
##
#########################################################
#########################################################
sub get_time_from_row {
    my ($row) = @_;
    if (ref($row)) {
        return $row->[&Apache::loncoursedata::RD_timestamp()];
    } 
    return undef;
}

sub get_tries_from_row {
    my ($row) = @_;
    if (ref($row)) {
        return $row->[&Apache::loncoursedata::RD_tries()];
    }
    return undef;
}

sub hashify_attempt {
    my ($row) = @_;
    my %attempt;
    $attempt{'tries'}      = $row->[&Apache::loncoursedata::RD_tries()];
    $attempt{'submission'} = $row->[&Apache::loncoursedata::RD_submission()];
    $attempt{'award'}      = $row->[&Apache::loncoursedata::RD_awarddetail()];
    $attempt{'timestamp'}  = $row->[&Apache::loncoursedata::RD_timestamp()];
    return %attempt;
}

sub Process_OR_Row {
    my ($row) = @_;
    my %RowData;
    my $student_id = $row->[&Apache::loncoursedata::RD_student_id()];
    my $award      = $row->[&Apache::loncoursedata::RD_awarddetail()];
    my $grading    = $row->[&Apache::loncoursedata::RD_response_eval()];
    my $submission = $row->[&Apache::loncoursedata::RD_submission()];
    my $time       = $row->[&Apache::loncoursedata::RD_timestamp()];
    my $tries      = $row->[&Apache::loncoursedata::RD_tries()];
    return undef if ($award eq 'MISSING_ANSWER');
    if ($award =~ /(APPROX_ANS|EXACT_ANS)/) {
        $RowData{'_correct'} = 1;
    }
    $RowData{'_total'} = 1;
    my @Foilgrades = split('&',$grading);
    my @Foilsubs   = split('&',$submission);
    for (my $j=0;$j<=$#Foilgrades;$j++) {
        my ($foilid,$correct)  = split('=',$Foilgrades[$j]);
        $foilid = &Apache::lonnet::unescape($foilid);
        my (undef,$submission) = split('=',$Foilsubs[$j]);
        if ($correct) {
            $RowData{$foilid}->{'_correct'}++;
        } else {
            $submission = &Apache::lonnet::unescape($submission);
            $RowData{$foilid}->{$submission}++;
        }
        $RowData{$foilid}->{'_total'}++;
    }
    return %RowData;
}

1;

__END__
